%%% PROGRAM FOR SPECTRALLY RESOLVING SHARP LINES %%%

%Spectral imaging program that automatically separates the signal from sharp lines
% from a background signal, and
%displays the resolved images: A gray image based on the resolved signal
%with sharp lines and an RGB image where the background is mapped in the
%green channel. The channel limits can be modified
%An image based on the signal gradient can
%also be displayed. The spectra along a straight line (or from a point) can
%be displayed

%Miguel R. Carro-Temboury%

close all;
clear all;

%% Datafile importing

%Spectral image directory and filename.
datapath='C:\Users\Tom_C410\Desktop\Cross-talk\Raw data\Crosstalk26072016\';
dataname='S7_z2Eu_465_LowRight_5mic_10pix';
%The spectral data is recorded in left-right , right-left mode and saved as a .txt file.
%The exact file format may vary slightly depending on the headers. Please check and customize the section
%"Importing data" in the code if necessary. In this case the .txt file
%contained some lines of text headers and two lines of numbers that will be
%ignored.
%The intensity data will be organized in an mxn matrix "J" containing intensity values where
%the spatial pixels are sorted by rows m (as recorded in time), and the
%corresponding CCD pixels are organized in the columns n.

%Starting row.
firstrow=3;

%Calibration file directory and filename.
calibrationpath='C:\Users\Tom_C410\Desktop\Cross-talk\Raw data\Crosstalk26072016\';
calibrationname='Calibrated_26072016_neon_300gr_680c';
%Calibration of the CCD array, wavelength against pixel. No headers, first
%column pixel and second column wavelength. The wavelength axes will be
%stored in the matrix "J2"

%Saving directory and filename
savepath='C:\Users\Tom_C410\Desktop\Cross-talk\Raw data\Crosstalk26072016\';
savename='Data1';
%Directory to save images

%% Scan size and resolution
%Resolution of the image in pixels
% rows y
mmax=10
% columns x
nmax=10
%Length and width of the image in microns
horscale=5;verscale=5;

%% Program options.
%'S'=No bakground subtraction. 'X'=Built in subtraction.
%'L1'=Image from absolute value of the gradient. 'L2'=Lanthanide resolved
%signal image
ProgramOption='L2';

% Images to be displayed '1' displayed '0' not displayed
% Sum of the spectra from the signal (S,X,L1 or L2)
TotalSpectra=1;

% Gray scale spectrally resolved image from the signal (S,X,L1 or L2)within
% the specified spectral channels (next section of parameters)
SpectrallyResolvedImage=1;

%Plots the spectra of the specified coordinates in "Line scan options"
% This also displays an auxiliary image to find the pixels from where the
% spectra is plotted
SpectraLine=1;

%Displays an RGB image specified by the channels in the next section
RGBImage=1;

%% Spectral channels options
% Range of spectral channels in pixel
% Check I2 variable(Calibration of wavelength) to see the corresponding
% wavelengths.
% The counts inside the indicated limits in the channel will be integrated to form the images


%RGB active channels for the RGB image.
R_on=1;
G_on=1;
B_on=0;
%Select which channel will be ploted in the Gray scale image. R G or B
Ch='R';


%Red Channel
%Eu sharpest band
% Eu1=329;
% Eu2=417;
%Whole spectral range
Eu1=1;
Eu2=1340;

% Blue Channel
%Tb sharpest band
Tb1=17;
Tb2=76;
%Whole spectral range
% Tb1=1;
% Tb2=1340;

% Green Channel is set to the background (dye) emission in program L2, can
% be set to another gate for other programs
G1=1000;
G2=1200;

%% Line Scan options
%The x and y coordinates of the pixels of the image from which the spectra will be displayed. They
%must have the same length
xcoord=[1:10];
ycoord=[1:10];

%% Threshold options

%options for program 4. Automatic detection of peaks
%Threshold counts for considering a peak in the gradient
T1=5;

%Minimum wavenumber distance between two features (gradient peaks) to be
%considered the same band.
T2=0.02E4;

%Threshold to remove filter feature
T3=40;

%minimum total counts of a spectra.
T4=2000;

%% Cosmic rays
%Remove cosmic ray. Set to 1 to remove
CosmicRemove=0;
% the cosmic ray with a gradient value above the threshold will be removed
cosmicthres=500;

%% Other options

%Smooth the data in the analysis when setting to one. otherwise 0. Needed
%to compute the gradient accurately.
SmoothOption=1;

%Individual normalization of RGB matrixes if set to 1. Otherwise to
%brightest pixel of the three matrixes.
indnorm=1;

%ccd background. Subtract dark counts from the CCD camera in the spectra. 'L2' signal is
% independent of this number and 'L1' is highly independent. The signals 'S' and 'X' are
% highly dependent on this background subtraction.
back=100;

%Manual gradient or built-in. Leave at one for built in
Builtingrad=0;

%SmoothGradient
SmoothGradient= 1;

%%
%%------------------------------code%%-------------------------------------%%

%% Importing data
I = importdata([datapath dataname '.txt']);
I2 = importdata([calibrationpath calibrationname '.txt']);

% clean data matrix with detector counts. Row represents spatial pixel and
% column CCD pixel.
%in this example two rows are ignored and no columns
J=I.data(firstrow:mmax*nmax+firstrow-1,1:end);
%example without ignored rows or columns
% J=I.data(1:mmax*nmax,1:end);
%In this case I is a structure array where I.text contains the text header
%lines and I.data containes the numbers in the .txt file

%If the datafile does not contain header lines, 'I' will not be a structure
%array, then one just needs to remove '.data'.
% J=I(1:mmax*nmax,1:end);

%wavelength axes
J2=I2(:,2);

%% background subtraction of a constant. back=0 by default
J=J-back;
J(J<0)=0;
%% Smooth data

if SmoothOption==1
    for l=1:size(J,1)
        smoothdat(l,:)=smooth(J(l,:),7,'sgolay',1);
    end
else
    smoothdat=J;
end

%% Gradient
%gradient matlab or manual
if ProgramOption=='L1' | ProgramOption=='L2'
    
    
    for i=1:size(smoothdat,1)
        for j=2:size(smoothdat,2)
            gradsmoothdat(i,j)=smoothdat(i,j)-smoothdat(i,j-1);
        end
    end
    
    %smooth the gradient
    if SmoothGradient==1
        for l=1:size(J,1)
            smoothgrad(l,:)=smooth(gradsmoothdat(l,:),7,'sgolay',1);
        end
        gradsmoothdat=smoothgrad;
    end
    abs_gradsmoothdat=abs(gradsmoothdat);
    abs_gradsmoothdat_tot=sum(abs_gradsmoothdat);
    gradsmoothdat_tot=sum(gradsmoothdat);
end

%% Remove Cosmic Ray
if ProgramOption=='L1'
    if CosmicRemove==1
        gradsmoothdat(gradsmoothdat>cosmicthres)=0;
        gradsmoothdat(gradsmoothdat<-cosmicthres)=0;
    end
end
%% Subtract background
if ProgramOption~='S'
    if ProgramOption=='X'
        for l=1:size(J,1)
            Yout(l,:)=msbackadj([1:1340]',smoothdat(l,:)');
        end
        smoothdat=Yout;
    end
    
    if ProgramOption=='L2'
        %The function AutoBackgroundRemove_2p3 carries out the background subtraction. To do debugging or
        %looking at particular traces, use figopt=1 or figopt=2 to display
        %auxiliary figures. figopt=2 are shown in the manuscript.  In that case use only one value in the for loop
        figopt=2;
        %normal for loop
        %         for l=1:size(J,1)
        %one value only for loop for debugging or display
        for l=47
            
            
            [Yout(l,:) BackDye(l,:)]=AutoBackgroundRemove_2p3(smoothdat(l,:),gradsmoothdat(l,:),J2,T1,T2,T3,T4,CosmicRemove,cosmicthres,figopt,savepath);
        end
        smoothdat=Yout;
        BackDye_tot=sum(BackDye);
    end
    
    smoothdat(smoothdat<0)=0;
end

J_tot=sum(J);
smoothdat_tot=sum(smoothdat);

%% Sum inside the spectral channels

if ProgramOption=='L1'
    A1=sum(abs_gradsmoothdat(:,Eu1:Eu2),2);
    B1=sum(abs_gradsmoothdat(:,G1:G2),2);
    C1=sum(abs_gradsmoothdat(:,Tb1:Tb2),2);
else
    A1=sum(smoothdat(:,Eu1:Eu2),2);
    if ProgramOption=='L2'
        B1=sum(BackDye(:,:),2);
    else
        B1=sum(smoothdat(:,G1:G2),2);
    end
    C1=sum(smoothdat(:,Tb1:Tb2),2);
end

A=A1;
B=B1;
C=C1;

if R_on==0
    A=A1*0;
end
if G_on==0
    B=B1*0;
end
if B_on==0
    C=C1*0;
end


%% Forming RGB images

for m = 1:mmax
    n=m*nmax;
    k=(m-1)*nmax+1;
    M1(m,:)=A(k:n);
    M2(m,:)=B(k:n);
    M3(m,:)=C(k:n);
end

for l=2:2:mmax
    M1(l,:)=fliplr(M1(l,:));
    M2(l,:)=fliplr(M2(l,:));
    M3(l,:)=fliplr(M3(l,:));
end

M1 = flipud (M1);
M2 = flipud (M2);
M3 = flipud (M3);

M(:,:,1)=M1;
M(:,:,2)=M2;
M(:,:,3)=M3;

if indnorm==0
    Mmi = min(M(:));
    Mma = max(M(:));
    M = (M-Mmi)./(Mma-Mmi);
else
    for i=1:3
        Mmi = min(min(M(:,:,i)));
        Mma = max(max(M(:,:,i)));
        M(:,:,i)=(M(:,:,i)-Mmi)./(Mma-Mmi);
    end
end

%% Line scan
%choosen coordinate or line scan (x coordinate values should be increasing)

for j=1:length(xcoord)
    x=xcoord(j);
    y=ycoord(j);
    
    if rem(mmax,2)==0 %even total number of rows
        if rem(y+1,2)==0 %odd index of row
            i=nmax*mmax-(y-1)*nmax-x+1;
            ind=1;
        else %even index of column
            i=nmax*mmax-y*nmax+x;
            ind=2;
        end
        
    else %odd total number of rows
        if rem(y+1,2)==0 %odd index of row
            i=nmax*mmax-y*nmax+x;
            ind=3;
        else %even index of column
            i=nmax*mmax-(y-1)*nmax-x+1;
            ind=4;
        end
    end
    
    Jline(:,j)=J(i,:);
    
    if ProgramOption=='L1'
        smoothdatline(:,j)=abs_gradsmoothdat(i,:);
    else
        smoothdatline(:,j)=smoothdat(i,:);
    end
end
%%
switch Ch
    case 'R'
        MG=M1;
    case 'G'
        MG=M2;
    case 'B'
        MG=M3;
end
%% Total Spectra Figure
if TotalSpectra==1
    
    h=figure;
    subpl_relpos(1,:)=[1/5 2/6 3/5 3/6];
    subpl_sizecm=[4.5 4.5];
    subpl_sizecm=[7 7];
    set(gcf, 'PaperPosition', Printer_subplot_Mig(subpl_relpos,subpl_sizecm));
    subplot('Position',subpl_relpos(1,:))
    
    %Uncomment to plot the total spectra with colored areas marking lanthanide
    %and background contribution
    % hh=area(J2,[BackDye_tot' smoothdat_tot'])
    % hh(1).FaceColor = [0 1 0];
    % hh(2).FaceColor = [0 0 1];
    % hh(2).FaceColor = [1 0 0];
    
    %uncomment/comment to plot the total spectra
    plot(J2, smoothdat_tot,'Color','k','LineWidth',1)
    if ProgramOption=='L2'
        hold on
        plot(J2, BackDye_tot,'Color','g','LineWidth',1)
    end
    
    %uncomment/comment to plot lines overimposed to the graph
    % hold on
    % x = [J2(Tb1) J2(Tb1)];
    % y = [0  max(max(smoothdat_tot'))];
    % % y = [0  max(max(BackDye_tot' +smoothdat_tot'))];
    % line(x,y,'Color','blue','LineStyle','--')
    % x = [J2(Tb2) J2(Tb2)];
    % y = [0  max(max(smoothdat_tot'))];
    % % y = [0  max(max(BackDye_tot' +smoothdat_tot'))];
    % line(x,y,'Color','blue','LineStyle','--')
    %
    % hold on
    % x = [J2(Eu1) J2(Eu1)];
    % y = [0  max(max(smoothdat_tot'))];
    % % y = [0  max(max(BackDye_tot' +smoothdat_tot'))];
    % line(x,y,'Color','red','LineStyle','--')
    % x = [J2(Eu2) J2(Eu2)];
    % y = [0  max(max(smoothdat_tot'))];
    % % y = [0  max(max(BackDye_tot' +smoothdat_tot'))];
    % line(x,y,'Color','red','LineStyle','--')
    
    set(gca,'FontSize',10)
    axis tight
    xlabel('Wavelength (nm)')
    ylabel('Integrated counts')
    title (['Total Spectra from ' ProgramOption])
    
    % print(h,'-dtiff','-r300',[savepath 'ImageAAA'])
    
end

%% Plot Spectrally resolved IMAGE
if SpectrallyResolvedImage==1
    
    pixhor=nmax;pixver=mmax;
    hor=[1:nmax]*horscale/pixhor;ver=[1:mmax]*verscale/pixver;
    
    h=figure;
    % subpl_relpos(1,:)=[1/5 2/6 3/5 3/6];
    subpl_relpos(1,:)=[1/6 2/6 3/6 3/6];
    subpl_relpos(2,:)=[9/12 2/6 1/12 3/6];
    
    subpl_sizecm=[7 7];
    set(gcf, 'PaperPosition', Printer_subplot_Mig(subpl_relpos,subpl_sizecm));
    subplot('Position',subpl_relpos(1,:))
    image(hor,ver,MG,'CDataMapping','scaled');
    set(gca,'YDir','normal','FontSize',10)
    xlabel('x (\mum)')
    ylabel('y (\mum)')
    colormap gray
    
    colorbar('Position',subpl_relpos(2,:))
    title (['Spectral image based on signal ' ProgramOption])
    
    % print(h,'-dtiff','-r300',[savepath 'ImageSpectrallyResolved'])
    
end

%% Plot Line Scan. The image with pixel axes is displayed to select
if SpectraLine==1
    
    h=figure;
    
    subpl_relpos(1,:)=[1/5 2/6 3/5 3/6];
    subpl_sizecm=[7 7];
    set(gcf, 'PaperPosition', Printer_subplot_Mig(subpl_relpos,subpl_sizecm));
    subplot('Position',subpl_relpos(1,:))
    image(MG,'CDataMapping','scaled');
    set(gca,'YDir','normal','FontSize',10)
    xlabel('x (pixel)')
    ylabel('y (pixel)')
    colormap gray
    
    title (['Find here the pixel coordinates for the line scan'])
    
    h=figure;
    subpl_relpos(1,:)=[1/6 2/6 3/6 3/6];
    subpl_relpos(2,:)=[9/12 2/6 1/12 3/6];
    
    subpl_sizecm=[7 7];
    set(gcf, 'PaperPosition', Printer_subplot_Mig(subpl_relpos,subpl_sizecm));
    subplot('Position',subpl_relpos(1,:))
    plot(J2,smoothdatline,'LineWidth',0.8);
    set(gca,'FontSize',10)
    ylabel('Integrated counts')
    xlabel('Wavelength (nm)')
    axis tight
    title (['Line scan based on signal ' ProgramOption])
    
    % legend(num2str([xcoord' ycoord']),'Location','eastoutside')
    % legend(num2str([xcoord' ycoord']),'Location','none','Position',subpl_relpos(2,:))
    legend({num2str([xcoord' ycoord'])},'Position',subpl_relpos(2,:))
    annotation('textbox',[8.5/12 3.6/12 2/12 7/12],'String','Pixel x,y ','FitBoxToText','off','Linestyle','none')
    
    % print(h,'-dtiff','-r300',[savepath 'ImageSpectrallyResolved'])
    
end

%% Plot RGB IMAGE
if RGBImage==1
    
    pixhor=nmax;pixver=mmax;
    hor=[1:nmax]*horscale/pixhor;ver=[1:mmax]*verscale/pixver;
    
    h=figure;
    subpl_relpos(1,:)=[1/5 2/6 3/5 3/6];
    subpl_sizecm=[7 7];
    set(gcf, 'PaperPosition', Printer_subplot_Mig(subpl_relpos,subpl_sizecm));
    subplot('Position',subpl_relpos(1,:))
    image(hor,ver,M,'CDataMapping','scaled');
    set(gca,'YDir','normal','FontSize',10)
    xlabel('x (\mum)')
    ylabel('y (\mum)')
    title ([' RGB Spectral image based on signal ' ProgramOption])
    
    % print(h,'-dtiff','-r300',[savepath 'ImageRGB'])
    
end